import json
import os

import logging

from service_buddy.code.cookie_cutter_creator import CookeCutterProjectCreator


class CodeCreator(object):
    def __init__(self, code_template_directory, dry_run):
        super(CodeCreator, self).__init__()
        default_path = os.path.join(code_template_directory, "code-template-config.json")
        if os.path.exists(default_path):
            with open(default_path) as fp:
                defaults = json.load(fp)
                self.default_provider = defaults.get('provider', None)
                self.extended_templates = defaults.get('code-template-definitions', {})
        else:
            logging.warn("Could not local 'code-template-config.json' in code template directory")
            self.default_provider = "cookiecutter"
            self.extended_templates ={}
        built_in = self._load_service_templates(
            os.path.join(os.path.dirname(os.path.abspath(__file__)), 'builtin_service_templates.json'))
        built_in.update(self.extended_templates)
        self.code_creators = {
            CookeCutterProjectCreator.get_type(): CookeCutterProjectCreator(template_dir=code_template_directory,
                                                                            dry_run=dry_run,
                                                                            templates=built_in)}
        if self.default_provider not in self.code_creators:
            raise Exception("Requested provider is not configured {}".format(self.default_provider))

    def _load_service_templates(self, builtIn):
        # type: (str) -> dict
        with open(builtIn) as builtin:
            return json.load(builtin)

    def get_default_code_creator(self):
        # type: () -> CodeCreator
        return self.code_creators[self.default_provider]

    def create_project(self, service_definition, service_dir):
        return self.get_default_code_creator().create_project(service_definition=service_definition,
                                                              service_dir=service_dir)



import click

from service_buddy.code.code_creator import CodeCreator
from service_buddy.commandline import cli
from service_buddy.context.service_context import ServiceContext
from service_buddy.service.service import Service


@cli.command(name='bootstrap',short_help="Create a repository containing the initial definition for a micro-service stack managed by service-buddy.")
@click.option("--application", envvar='APPLICATION', required=True,
              help='Application name for new micro-service stack.  Suggest brevity.')
@click.pass_obj
def bootstrap(service_ctx,application):
    # type: (ServiceContext, str) -> None
    cc = CodeCreator("", service_ctx.dry_run)
    service_def = Service(app=application,role="master",definition={})
    service_def.set_service_type("service-buddy-master")
    cc.create_project(service_definition=service_def,service_dir=service_ctx.destination_directory)